/**************************************************************

File Name:		Vector_Maximum.asm
		
Date Modified:	02/17/99	RFG		original version
				05/01/00	RFG		modified format
				07/13/00	RFG		verified on silicon

Description:
			Subroutine that records the value and location of
			the maximum value in a given (real) vector.

			Equation:		MAX_VAL = MAX[INPUT]
								MAX_INDEX = i given MAX_VAL = INPUT(i)

Calling Parameters:
			b0,i0 = address of input data
			l0 = 0
			r1 = number of samples / 4
			
Assumptions:
			All arrays must start on even normal-word address boundaries.
			All arrays must have a multiple of 4 length (pad with zeros if necessary).
			The integer values 4 & 3 should be appended to the 
			end of the input file as offsets to find MAX_INDEX.
			

Return Values:
			f4=MAX_VAL
			f2=MAX_INDEX

Registers Affected:
			f0,f2,f4,f8
			i0

Cycle Count:
			12 + N
			
Number of PM Locations:
			16 instruction words
									
Number of DM Locations:
			N  for the Number of elements for the vector INPUT
			2  for MAX_VALUE
			2  for MAX_INDEX

**************************************************************/

#include    "def21364.h"

.global vec_max;

/* program memory code */
.section/pm seg_pmco;

vec_max:
/* alu, multiplier precision, SIMD mode enable */
    bit set MODE1 RND32 | PEYEN;
	nop;
	
    f4=dm(i0,2);
    f0=pass f4;
    lcntr=r1, do vecmax until lce;		/* vector maximum loop */
	    	comp(f4,f0), f8=dm(i0,2);	/* last read is appended offset for vector location */
            if le f4=pass f0, r2=i0;
            comp(f4,f8), f0=dm(i0,2);
vecmax:	    if le f4=pass f8, r2=i0;

	r2=r2-r0, r0=b0;
    r2=r2-r0, r0=s4;
    comp(f4,f0);
    if le f4=pass f0, r2<->s2;

    rts (db);
	bit clr MODE1 PEYEN;
vec_max.end:	nop;

